 

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Implementazione di query jdbc su una versione hqldb della tabella prodotto
 * 
 * La scrittura dei metodi DAO con JDBC segue sempre e comunque questo schema:
 * 
	// --- 1. Dichiarazione della variabile per il risultato ---
	// --- 2. Controlli preliminari sui dati in ingresso ---
	// --- 3. Apertura della connessione ---
	// --- 4. Tentativo di accesso al db e impostazione del risultato ---
	// --- 5. Gestione di eventuali eccezioni ---
	// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
	// --- 7. Restituzione del risultato (eventualmente di fallimento)
 * 
 * Inoltre, all'interno di uno statement JDBC si segue lo schema generale
 * 
	// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
	// --- b. Pulisci e imposta i parametri (se ve ne sono)
	// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
	// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
	// --- e. Rilascia la struttura dati del risultato
	// --- f. Rilascia la struttura dati dello statement
 * 
 * @author pisi79
 *
 */
public class HsqldbProdottoDAO 
implements ProdottoDAO {

	// === Costanti letterali per non sbagliarsi a scrivere !!! ============================
	
	static final String prodotto_table = "prodotti";

	// -------------------------------------------------------------------------------------

	static final String prodotto_id = "id";
	static final String prodotto_name = "name";
	static final String prodotto_marca = "marca";
	static final String prodotto_prezzo = "prezzo";
	
	// == STATEMENT SQL ====================================================================

	// INSERT INTO prodotti ( namecolumn,surnamecolumn, ...) VALUES ( ?,?, ... );
	static final String insert = 
		"INSERT " +
			"INTO " + prodotto_table + " ( " + 
				prodotto_id +", "+prodotto_name+", "+prodotto_marca+", "+prodotto_prezzo+
			") " +
			"VALUES (?,?,?,?) ;"
		;
	
	// SELECT * FROM prodotto_table WHERE idcolumn = ?;
	static String read_by_id = 
		"SELECT * " +
			"FROM " + prodotto_table + " " +
			"WHERE " + prodotto_id + " = ? ;"
		;


	// SELECT * FROM prodotto_table;
	static String query = 
		"SELECT * " +
			"FROM " + prodotto_table + " ;"
		;

	// -------------------------------------------------------------------------------------

	// CALL IDENTITY();
	/**
	 * Restituisce l'identificativo dell'ultima tupla inserita. FUNZIONE NON CROSS-DATABASE!!!
	 */
	static String lastInsert = 
		"CALL " + 
			"IDENTITY() ;"
		;

	// -------------------------------------------------------------------------------------

	// CREATE prodotto_table ( idcolumn INT NOT NULL IDENTITY, namecolumn VARCHAR(50), ..., UNIQUE ( idcolumn ) );
	static String create = 
		"CREATE " +
			"TABLE " + prodotto_table +" ( " +
				prodotto_id + " VARCHAR(50), " +
				prodotto_name + " VARCHAR(50), " +
				prodotto_marca + " VARCHAR(50), " +
				prodotto_prezzo + " LONGVARCHAR, " +
				"UNIQUE( " + prodotto_id + " ) " +
			") ;"
		;
	static String drop = 
		"DROP " +
			"TABLE " + prodotto_table + " ;"
		;
	
	// === METODI DAO =========================================================================
	
	/**
	 * C
	 */
	public int createProdotto(ProdottoDTO t); {
		// --- 1. Dichiarazione della variabile per il risultato ---
		int result = -1;
		// --- 2. Controlli preliminari sui dati in ingresso ---
		if ( t == null )  {
			System.err.println(HsqldbProdottoDAO.class.getName()+".createProdotto(): failed to insert a null product");
			return result;
		}
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbProdottoDAO.insert);
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			prep_stmt.clearParameters();
			prep_stmt.setString(1, t.getId());
			prep_stmt.setString(2, t.getNome());
			prep_stmt.setString(3, t.getMarca());
			prep_stmt.setLong(4, t.getPrezzo());
		

			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			prep_stmt.executeUpdate();
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			// n.d.
			// --- e. Rilascia la struttura dati del risultato
			// n.d.
			// --- f. Rilascia la struttura dati dello statement
			prep_stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(HsqldbProdottoDAO.class.getName()+".createProdotto(): failed to insert product\n"+e);
			e.printStackTrace();
			result = -2;
		}

		// Nel caso della creazione di una nuova tupla eseguo un secondo accesso per sapere che id le è stato assegnato
		// Chiaramente è inutile farlo se già il primo accesso ha prodotto errori
		// --- 6./7. Chiusura della connessione in caso di errori e restituizione prematura di un risultato di fallimento
		if ( result == -2 ) {
			HsqldbDAOFactory.closeConnection(conn);
			return result;
		}
		
		// --- 1. Dichiarazione della variabile per il risultato ---
		// riutilizziamo quella di prima

		// --- 2. Controlli preliminari sui dati in ingresso ---
		// già fatti

		// --- 3. Apertura della connessione ---
		// ce n'è una già aperta, se siamo qui

		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(HsqldbProdottoDAO.lastInsert);
			if ( rs.next() ) {
				result = rs.getInt(1);
			}		
			rs.close();
			stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(HsqldbProdottoDAO.class.getNome()+".createProdotto(): failed to retrieve id of last inserted product\n"+e);
			e.printStackTrace();
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

	// -------------------------------------------------------------------------------------

	/**
	 * R
	 */
	public ProdottoDTO readProdotto(int id) {
		// --- 1. Dichiarazione della variabile per il risultato ---
		ProdottoDTO result = null;
		// --- 2. Controlli preliminari sui dati in ingresso ---
	 
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbProdottoDAO.read_by_id);
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, id);
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			ResultSet rs = prep_stmt.executeQuery();
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			if ( rs.next() ) {
				ProdottoDTO prod = new ProdottoDTO();
				prod.setId(rs.getString(prodotto_id));
				prod.setName(rs.getString(prodotto_name));
				prod.setSurname(rs.getString(prodotto_marca));
				prod.setAddress(rs.getString(prodotto_prezzo));
		
				result = prod;
			}
			// --- e. Rilascia la struttura dati del risultato
			rs.close();
			// --- f. Rilascia la struttura dati dello statement
			prep_stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(HsqldbProdottoDAO.class+".readProdotto(): failed to retrieve product with id = " + id+"\n"+e);
			e.printStackTrace();
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

	// -------------------------------------------------------------------------------------

	/**
	 * U
	 */
	public boolean updateProdotto(ProdottoDTO t) {
	 		return true;
	}

	// -------------------------------------------------------------------------------------

	/**
	 * D
	 */
	public boolean deleteProdotto(ProdottoDTO t) {
 		return true;
	}

	// -------------------------------------------------------------------------------------

	/**
	 * Find by nick
	 */
	public ProdottoDTO findProdotto(String id) {
		// --- 1. Dichiarazione della variabile per il risultato ---
		ProdottoDTO result = null;
		// --- 2. Controlli preliminari sui dati in ingresso ---


		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			PreparedStatement prep_stmt = conn.prepareStatement(HsqldbProdottoDAO.read_by_id);
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			prep_stmt.clearParameters();
			prep_stmt.setString(1, id);
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			ResultSet rs = prep_stmt.executeQuery();
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			if ( rs.next() ) {
				ProdottoDTO prod = new ProdottoDTO();
				prod.setId(rs.getString(prodotto_id));
				prod.setName(rs.getString(prodotto_name));
				prod.setSurname(rs.getString(prodotto_marca));
				prod.setAddress(rs.getString(prodotto_prezzo));
				result = prod;
			}
			// --- e. Rilascia la struttura dati del risultato
			rs.close();
			// --- f. Rilascia la struttura dati dello statement
			prep_stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(HsqldbProdottoDAO.class+".findProdotto(): failed to retrieve product with id = " + id+"\n"+e);
			e.printStackTrace();
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

	// -------------------------------------------------------------------------------------

	public Collection<ProdottoDTO> getProdotti() {
 	    Collection<ProdottoDTO> result = new ArrayList<ProdottoDTO>();
 		return result;
	}

	// -------------------------------------------------------------------------------------
	// -------------------------------------------------------------------------------------

	/**
	 * Creazione della table
	 */
	public boolean createTable() {
		// --- 1. Dichiarazione della variabile per il risultato ---
		boolean result = false;
		// --- 2. Controlli preliminari sui dati in ingresso ---
		// n.d.
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			Statement stmt = conn.createStatement();
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			// n.d.
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			stmt.execute(create);
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			// n.d. Qui devo solo dire al chiamante che è andato tutto liscio
			result = true;
			// --- e. Rilascia la struttura dati del risultato
			// n.d.
			// --- f. Rilascia la struttura dati dello statement
			stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(HsqldbProdottoDAO.class.getName()+".createTable(): failed to create prodotto table\n"+e);
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

	// -------------------------------------------------------------------------------------

	/**
	 * Rimozione della table
	 */
	public boolean dropTable() {
		// --- 1. Dichiarazione della variabile per il risultato ---
		boolean result = false;
		// --- 2. Controlli preliminari sui dati in ingresso ---
		// n.d.
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			Statement stmt = conn.createStatement();
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			// n.d.
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			stmt.execute(drop);
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			// n.d. Qui devo solo dire al chiamante che è andato tutto a posto.
			result = true;
			// --- e. Rilascia la struttura dati del risultato
			// n.d.
			// --- f. Rilascia la struttura dati dello statement
			stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(HsqldbProdottoDAO.class.getName()+".dropTable(): failed to drop product table\n"+e);
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

}
